@@ -10,13 +10,17 @@ module Network.TLS
     (
     -- * Context configuration
       ClientParams(..)
+    , HostName
+    , Bytes
     , ServerParams(..)
     , DebugParams(..)
+    , DHParams
     , ClientHooks(..)
     , ServerHooks(..)
     , Supported(..)
     , Shared(..)
     , Hooks(..)
+    , Handshake
     , Logging(..)
     , Measurement(..)
     , CertificateUsage(..)
@@ -45,7 +49,7 @@ module Network.TLS
     , Context
     , ctxConnection
     , TLSParams
-    , HasBackend
+    , HasBackend(..)
 
     -- * Creating a context
     , contextNew
@@ -62,6 +66,9 @@ module Network.TLS
 
     -- * Information gathering
     , Information(..)
+    , ClientRandom
+    , ServerRandom
+
     , unClientRandom
     , unServerRandom
     , contextGetInformation
@@ -120,13 +127,15 @@ module Network.TLS
     , exceptionValidationCache
     ) where
 
-import Network.TLS.Backend (Backend(..), HasBackend)
+import Network.TLS.Backend (Backend(..), HasBackend(..))
 import Network.TLS.Struct ( TLSError(..), TLSException(..)
                           , HashAndSignatureAlgorithm, HashAlgorithm(..), SignatureAlgorithm(..)
                           , Header(..), ProtocolType(..), CertificateType(..)
                           , AlertDescription(..)
-                          , ClientRandom(..), ServerRandom(..))
-import Network.TLS.Crypto (KxError(..))
+                          , ClientRandom(..), ServerRandom(..)
+                          , Bytes
+                          , Handshake)
+import Network.TLS.Crypto (KxError(..), DHParams)
 import Network.TLS.Cipher
 import Network.TLS.Hooks
 import Network.TLS.Measurement
