@@ -81,6 +81,7 @@ import Data.Maybe (isJust)
 import Control.Concurrent.MVar
 import Control.Monad.State
 import Data.IORef
+import Data.Monoid (mappend)
 
 -- deprecated imports
 #ifdef INCLUDE_NETWORK
@@ -91,7 +92,7 @@ import System.IO (Handle)
 class TLSParams a where
     getTLSCommonParams :: a -> CommonParams
     getTLSRole         :: a -> Role
-    getCiphers         :: a -> [Cipher]
+    getCiphers         :: a -> Credentials -> [Cipher]
     doHandshake        :: a -> Context -> IO ()
     doHandshakeWith    :: a -> Context -> Handshake -> IO ()
 
@@ -101,7 +102,7 @@ instance TLSParams ClientParams where
                                  , clientDebug cparams
                                  )
     getTLSRole _ = ClientRole
-    getCiphers cparams = supportedCiphers $ clientSupported cparams
+    getCiphers cparams _ = supportedCiphers $ clientSupported cparams
     doHandshake = handshakeClient
     doHandshakeWith = handshakeClientWith
 
@@ -113,7 +114,7 @@ instance TLSParams ServerParams where
     getTLSRole _ = ServerRole
     -- on the server we filter our allowed ciphers here according
     -- to the credentials and DHE parameters loaded
-    getCiphers sparams = filter authorizedCKE (supportedCiphers $ serverSupported sparams)
+    getCiphers sparams extraCreds = filter authorizedCKE (supportedCiphers $ serverSupported sparams)
           where authorizedCKE cipher =
                     case cipherKeyExchange cipher of
                         CipherKeyExchange_RSA         -> canEncryptRSA
@@ -121,20 +122,24 @@ instance TLSParams ServerParams where
                         CipherKeyExchange_DHE_RSA     -> canSignRSA && canDHE
                         CipherKeyExchange_DHE_DSS     -> canSignDSS && canDHE
                         CipherKeyExchange_ECDHE_RSA   -> canSignRSA
-                        -- unimplemented: non ephemeral DH
+                        -- unimplemented: EC
+                        CipherKeyExchange_ECDHE_ECDSA -> False
+                        -- unimplemented: non ephemeral DH & ECDH.
+                        -- Note, these *should not* be implemented, and have
+                        -- (for example) been removed in OpenSSL 1.1.0
+                        --
                         CipherKeyExchange_DH_DSS      -> False
                         CipherKeyExchange_DH_RSA      -> False
-                        -- unimplemented: EC
                         CipherKeyExchange_ECDH_ECDSA  -> False
                         CipherKeyExchange_ECDH_RSA    -> False
-                        CipherKeyExchange_ECDHE_ECDSA -> False
 
                 canDHE        = isJust $ serverDHEParams sparams
                 canSignDSS    = SignatureDSS `elem` signingAlgs
                 canSignRSA    = SignatureRSA `elem` signingAlgs
                 canEncryptRSA = isJust $ credentialsFindForDecrypting creds
                 signingAlgs   = credentialsListSigningAlgorithms creds
-                creds         = sharedCredentials $ serverShared sparams
+                serverCreds   = sharedCredentials $ serverShared sparams
+                creds         = extraCreds `mappend` serverCreds
     doHandshake = handshakeServer
     doHandshakeWith = handshakeServerWith
 
@@ -159,7 +164,9 @@ contextNew backend params = liftIO $ do
         st   = newTLSState rng role
         ciphers = getCiphers params
 
-    when (null ciphers) $ error "no ciphers available with those parameters"
+--  we still might get some ciphers from SNI callback
+--  If we could bail only on protocols which require the main cert??
+--    when (null (ciphers mempty)) $ error "no ciphers available with those parameters"
 
     stvar <- newMVar st
     eof   <- newIORef False
