@@ -88,6 +88,8 @@ data ClientParams = ClientParams
     , clientWantSessionResume         :: Maybe (SessionID, SessionData)
     , clientShared                    :: Shared
     , clientHooks                     :: ClientHooks
+      -- | In this element, you'll  need to override the default empty value of
+      -- of 'supportedCiphers' with a suitable cipherlist.
     , clientSupported                 :: Supported
     , clientDebug                     :: DebugParams
     } deriving (Show)
@@ -144,12 +146,21 @@ data Supported = Supported
       -- On the client side, the highest version will be used to establish the connection.
       -- On the server side, the highest version that is less or equal than the client version will be chosed.
       supportedVersions       :: [Version]
-      -- | Supported cipher methods
+      -- | Supported cipher methods.  The default is empty, specify a suitable
+      -- cipher list.  'Network.TLS.Extra.Cipher.ciphersuite_default' is often
+      -- a good choice.
     , supportedCiphers        :: [Cipher]
       -- | supported compressions methods
     , supportedCompressions   :: [Compression]
       -- | All supported hash/signature algorithms pair for client
-      -- certificate verification, ordered by decreasing priority.
+      -- certificate verification and server signature in (EC)DHE,
+      -- ordered by decreasing priority.
+      --
+      -- This list is sent to the peer as part of the signature_algorithms
+      -- extension.  It is also used to restrict the choice of hash and
+      -- signature algorithm, but only when the TLS version is 1.2 or above.
+      -- In order to disable SHA-1 one must then also disable earlier protocol
+      -- versions in 'supportedVersions'.
     , supportedHashSignatures :: [HashAndSignatureAlgorithm]
       -- | Secure renegotiation defined in RFC5746.
       --   If 'True', clients send the renegotiation_info extension.
@@ -182,9 +193,11 @@ defaultSupported = Supported
     , supportedCiphers        = []
     , supportedCompressions   = [nullCompression]
     , supportedHashSignatures = [ (Struct.HashSHA512, SignatureRSA)
+                                , (Struct.HashSHA512, SignatureECDSA)
                                 , (Struct.HashSHA384, SignatureRSA)
+                                , (Struct.HashSHA384, SignatureECDSA)
                                 , (Struct.HashSHA256, SignatureRSA)
-                                , (Struct.HashSHA224, SignatureRSA)
+                                , (Struct.HashSHA256, SignatureECDSA)
                                 , (Struct.HashSHA1,   SignatureRSA)
                                 , (Struct.HashSHA1,   SignatureDSS)
                                 ]
@@ -240,6 +253,10 @@ data ClientHooks = ClientHooks
                                Maybe [HashAndSignatureAlgorithm],
                                [DistinguishedName]) -> IO (Maybe (CertificateChain, PrivKey))
     , onNPNServerSuggest   :: Maybe ([B.ByteString] -> IO B.ByteString)
+      -- | Used by the client to validate the server certificate.  The default
+      -- implementation calls 'validateDefault' which validates according to the
+      -- default hooks and checks provided by "Data.X509.Validation".  This can
+      -- be replaced with a custom validation function using different settings.
     , onServerCertificate  :: CertificateStore -> ValidationCache -> ServiceID -> CertificateChain -> IO [FailedReason]
     , onSuggestALPN :: IO (Maybe [B.ByteString])
     }
@@ -266,9 +283,8 @@ data ServerHooks = ServerHooks
       onClientCertificate :: CertificateChain -> IO CertificateUsage
 
       -- | This action is called when the client certificate
-      -- cannot be verified.  A 'Nothing' argument indicates a
-      -- wrong signature, a 'Just e' message signals a crypto
-      -- error.
+      -- cannot be verified. Return 'True' to accept the certificate
+      -- anyway, or 'False' to fail verification.
     , onUnverifiedClientCert :: IO Bool
 
       -- | Allow the server to choose the cipher relative to the
