@@ -6,7 +6,7 @@
 -- Portability : unknown
 --
 -- the Sending module contains calls related to marshalling packets according
--- to the TLS state 
+-- to the TLS state
 --
 module Network.TLS.Sending (writePacket) where
 
@@ -72,7 +72,9 @@ prepareRecord ctx f = do
     txState <- readMVar $ ctxTxState ctx
     let sz = case stCipher $ txState of
                   Nothing     -> 0
-                  Just cipher -> bulkIVSize $ cipherBulk cipher
+                  Just cipher -> if hasRecordIV $ bulkF $ cipherBulk cipher
+                                    then bulkIVSize $ cipherBulk cipher
+                                    else 0 -- to not generate IV
     if hasExplicitBlockIV ver && sz > 0
         then do newIV <- getStateRNG ctx sz
                 runTxState ctx (modify (setRecordIV newIV) >> f)
@@ -86,5 +88,5 @@ switchTxEncryption ctx = do
                                       return (v, c)
     liftIO $ modifyMVar_ (ctxTxState ctx) (\_ -> return tx)
     -- set empty packet counter measure if condition are met
-    when (ver <= TLS10 && cc == ClientRole && isCBC tx) $ liftIO $ writeIORef (ctxNeedEmptyPacket ctx) True
+    when (ver <= TLS10 && cc == ClientRole && isCBC tx && supportedEmptyPacket (ctxSupported ctx)) $ liftIO $ writeIORef (ctxNeedEmptyPacket ctx) True
   where isCBC tx = maybe False (\c -> bulkBlockSize (cipherBulk c) > 0) (stCipher tx)
