@@ -57,7 +57,7 @@ module Network.TLS.State
     , withRNG
     ) where
 
-import Control.Applicative
+import Network.TLS.Imports
 import Network.TLS.Struct
 import Network.TLS.RNG
 import Network.TLS.Types (Role(..))
